#!/bin/bash
##############################################################################
# audioMMT
# Software to extract and convert MeTaLMaDNeSS.TeaM releases to several other
# formats.
#
# Returns:
# 0 - Everything went OK
# 1 - Failed to detect release type, probably a bug in audiommt.sh
# 2 - There was no preferences file, default created
# 3 - Invalid operation mode requested, usage displayed
# 4 - Error during operation
#
# It requires the following software. The number in parentheses is the 
# software version used for testing. Different versions might (or not) work.
# Particularly, older versions of shntools will not work properly:
#
# bash:     For executing the commands in the script. (3.2.9)
# cuetools: For processing audio files. 
#           Get it from http://developer.berlios.de/projects/cuetools/
# flac:     For compressing/uncompressing FLAC files. (1.1.4)
# id3:      For tagging the resulting MP3 files. (0.78)
#           Get it from http://home.wanadoo.nl/squell/id3.html
# lame:     For compressing on MP3. (3.97)
# mac:      For compressing/uncompresing APE files. (3.99u4b5)
# mppenc:   For compressing on MPC. (1.16)
#           Get it from http://www.musepack.net
# oggenc:   For compressing on OGG. (1.1.1)
# sed:      It works with the GNU one, not tested with any other. (4.1.5)
# shntool:  For processing audio files. (3.0.3)
#           Get it from http://freshmeat.net/projects/shntool/
# unace:    For uncompressing ACE releases (2.5)
#           Get it from http://www.winace.com/
# unrar:    For uncompressing RAR releases (3.70 beta 1)
#           Get it from http://www.rarsoft.com/
# unzip:    For uncompressing ZIP.WV releases (5.52)
# wavpack:  For compressing WV files (4.41.0)
# wvunpack: For uncompressing WV files (4.41.0)
##############################################################################

##############################################################################
# cleanup function
#
# It deletes the temporary folder used during conversion.
##############################################################################
cleanup ()
{
	echo ""
	echo "Cleaning up temporary files"
	cd "${TEMP_DIR}"/..
	rm -rf "${TEMP_DIR}"
} 

##############################################################################
# usage function
#
# It shows the usage instructions
##############################################################################
usage ()
{
	echo " "
	echo "`basename $0`"
	echo "Usage: `basename $0` <MODE> <MMT_FILE> [<MMT_FILE>*]"
	echo " "
	echo "<MODE> Should be one of:"
	echo "   -mp3     : Use it to convert the release to individual mp3 files"
	echo "   -ogg     : Use it to convert the release to individual ogg files"
	echo "   -mpc     : Use it to convert the release to individual mpc files"
	echo "   -ape     : Use it to convert the release to individual ape files"
	echo "   -flac    : Use it to convert the release to individual flac files"
	echo "   -wv      : Use it to convert the release to individual wv files"
	echo "<MMT_FILE>  : RAR, ZIP or ACE file containing an MMT release."
	echo "              It can also be a folder with an uncompressed release"
	echo "              There can be as many as needed, all will be processed"
	echo "              the same way as specified by <MODE>"
	echo " "
}

##############################################################################
# moveExtraFiles function
#
# It moves the extra files (like rip log, scans, etc) to the destination folder
##############################################################################
moveExtraFiles ()
{
	echo ""
	echo "Moving extra files to ${DEST_DIR}/$ALBUM_ARTIST/$ALBUM_SUBDIR"
	mv "${TEMP_DIR}"/* "${DEST_DIR}/$ALBUM_ARTIST/$ALBUM_SUBDIR" &>/dev/null
}

##############################################################################
# releasetype function
#
# It tells the script the kind of release it's dealing with.
#
# Release types are:
# 1 - One image, one CUE file (normal release)
# 2 - One image, more than once CUE file
# 3 - More than one image, one CUE file per image (normal release, various CDs)
# 4 - More than one image, more than one CUE file per image
# 5 - Separated tracks release (no CUE)
# 6 - Other (?)
##############################################################################
releasetype ()
{
	# Count the number of cue files found
	NUMBER_OF_CUES=`ls *.cue 2>/dev/null | grep -c .`

	# Count the total number of audio files	
	NUMBER_OF_IMAGES=`ls *.ape *.flac *.wv *.wav 2>/dev/null | grep -c .`
	
	if [[ $NUMBER_OF_CUES -eq $NUMBER_OF_IMAGES && $NUMBER_OF_CUES -eq 1 ]]; then #Standard 1CD releases
		echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 1"
		return 1
	elif [[ $NUMBER_OF_CUES -eq $NUMBER_OF_IMAGES && $NUMBER_OF_CUES -gt 1 ]]; then #Standard multiCD releases
		echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 3"
		return 3
	elif [[ $NUMBER_OF_CUES -gt $NUMBER_OF_IMAGES && $NUMBER_OF_IMAGES -eq 1 ]]; then #1CD Release, several cues
		echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 2"
		return 2
	elif [[ $NUMBER_OF_CUES -gt $NUMBER_OF_IMAGES && $NUMBER_OF_IMAGES -gt 1 ]]; then #multiCD release, several cues per image
		echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 4"
		return 4
	elif [[ $NUMBER_OF_IMAGES -gt 0 ]]; then #It has less cues than audio files, probably a separated tracks release
		echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 5"
		return 5
	else #Unknown type of release, most probably not a valid release
		echo "Found $NUMBER_OF_CUES CUE and $NUMBER_OF_IMAGES IMAGE, release type 6"
		return 6
	fi
}

##############################################################################
# compress function
#
# It compresses the supplied WAV file to the specified format.
#
# First parameter should be the WAV filename to compress (with extension)
# Second parameter should be the destination filename (with path if needed)
# Third parameter should be the format it should be compressed to
# (one of ape,flac,mp3,mpc,wv)
##############################################################################
compress ()
{
	if [ $3 = "mp3" ]; then
		#Compress to MP3 the current WAV file
		lame $LAME_OPTIONS "$1" "$2"
	elif [ $3 = "mpc" ]; then
		#Compress to MPC the current WAV file
		mppenc $MPPENC_OPTIONS "$1" "$2"
	elif [ $3 = "ogg" ]; then
		#Compress to MPC the current WAV file
		oggenc $OGGENC_OPTIONS "$1" -o "$2"
	elif [ $3 = "ape" ]; then
		#Compress to APE the current WAV file
		mac "$1" "$2" $MAC_OPTIONS
	elif [ $3 = "flac" ]; then
		#Compress to FLAC the current WAV file
		flac $FLAC_OPTIONS "$1" "$2"
	elif [ $3 = "wv" ]; then
		#Compress to WV the current WAV file
		wavpack $WAVPACK_OPTIONS "$1" "$2"
	fi
	return $?
}

##############################################################################
# tag function
#
# Tags an audio file with the supplied data (if possible)
# First parameter should be the file to tag
# Second parameter should be the TRACK_TITLE
# Third parameter should be the ALBUM_ARTIST
# Fourth parameter should be the ALBUM_TITLE
# Fifth parameter should be the ALBUM_YEAR
# Sixth parameter should be the TRACK_NUMBER
# Seventh parameter should be the file type (one of ape,flac,mp3,mpc,wv)
##############################################################################
tag ()
{
	if [ $7 = "mp3" ]; then
		#Tag the given MP3 file. It will fill ID3v1 and ID3v2 tags.
		id3 -12 -t "$2" -a "$3" -l "$4" -y "$5" -n "$6" "$1"
	elif [ $7 = "mpc" ]; then
		#Tag the given MPC file.
		echo "No tag yet for mpc files"
	elif [ $7 = "ogg" ]; then
		#Tag the given OGG file.
		vorbiscomment -a "$1" -t "TITLE=$2" -t "ARTIST=$3" -t "ALBUM=$4" -t "DATE=$5" -t "TRACKNUMBER=$6" 
	elif [ $7 = "ape" ]; then
		#Tag the given APE file.
		echo "No tag yet for ape files"
	elif [ $7 = "flac" ]; then
		#Tag the given FLAC file.
		metaflac --set-tag="TITLE=$2" --set-tag="ARTIST=$3" --set-tag="ALBUM=$4" --set-tag="DATE=$5" --set-tag="TRACKNUMBER=$6" "$1"
	elif [ $7 = "wv" ]; then
		#Tag the given WV file.
		echo "No tag yet for wavpack files"
	fi
}

##############################################################################
# converttotracks function
#
# First parameter should be the CUE filename (with extension)
# Second parameter should be the IMAGE filename (with extension)
# Third parameter should be the desired output extension
##############################################################################
converttotracks ()
{
	#Get album information and create destination folder
	ALBUM_TITLE=`cueprint -d %T "$1" | sed 's/\//_/g'`
	ALBUM_ARTIST=`cueprint -d %P "$1" | sed 's/\//_/g'`
	ALBUM_SUBDIR="${ALBUM_PREFIX}${ALBUM_TITLE}"
	ALBUM_YEAR="${YEAR}"
	TRACKS=`cueprint -d %N "$1"`
	
	mkdir -p "${DEST_DIR}/${ALBUM_ARTIST}/${ALBUM_SUBDIR}"

	echo "This album contains ${TRACKS} tracks"
	# Check if the release has more than one track. If only one, shnsplit does not behave properly...
	if [ ! -z "$(cuebreakpoints "$1")" ]; then	
		#Split the IMAGE in WAV tracks acording to the CUE and then 
		#delete the IMAGE and the CUE. Every track will be named "split-trackNN.wav"
		cuebreakpoints "$1" | shnsplit -n %02d -o wav "$2"
	else
		shnconv -o wav "$2"
		mv "${2%.${INPUT_EXTENSION}}".wav split-track01.wav
	fi

	#If the release was correctly split, continue
	if [ $? -eq 0 ]; then
		rm -f "$2"
	
		#Compress each of the WAVs resulting from splitting the IMAGE
		COMPRESS_ERRORS=0
		for I in `seq 1 ${TRACKS}`
		do
			TRACK_TITLE=`cueprint -t %t -n $I "$1" | sed 's/\//_/g'`
			if (( I < 10 )); then
				TRACK_NUMBER=0$I
			else
				TRACK_NUMBER=$I
			fi

			compress "split-track${TRACK_NUMBER}.wav" "${DEST_DIR}/${ALBUM_ARTIST}/${ALBUM_SUBDIR}/${TRACK_NUMBER} - ${TRACK_TITLE}".${3} ${3}
			if [ $? -ne 0 ]; then
				let COMPRESS_ERRORS+=1
			fi
			tag "${DEST_DIR}/${ALBUM_ARTIST}/${ALBUM_SUBDIR}/${TRACK_NUMBER} - ${TRACK_TITLE}".${3} "${TRACK_TITLE}" "${ALBUM_ARTIST}" "${ALBUM_TITLE}" "${ALBUM_YEAR}" "${TRACK_NUMBER}" ${3}
			rm -f "split-track${TRACK_NUMBER}.wav"
		done
		if [ $COMPRESS_ERRORS -gt 0 ]; then
			error "There were ${COMPRESS_ERRORS} tracks that had errors while compressing"
		else
			let SUCCESS+=1
		fi
		rm -f "$1"
	else
		error "Problem converting the release"
	fi
}

##############################################################################
# process function
#
# First parameter should be the CUE filename (with extension)
# Second parameter should be the IMAGE filename (with extension)
# Third parameter should be the conversion mode requested by the user
##############################################################################
process ()
{
	if [[ $3 = "mp3" || $3 = "mpc" || $3 = "ogg" || $3 = "ape" || $3 = "flac" || $3 = "wv" ]]; then
		converttotracks "$1" "$2" "$3"
	else
		usage
		exit 1
	fi
}
##############################################################################
# checkinputformat function
#
# It will check the audio format used to compress the release, and set the INPUT_EXTENSION
# variable accordingly.
##############################################################################
checkinputformat ()
{
	if ls *.ape &>/dev/null; then
		INPUT_EXTENSION=ape
	fi
	if ls *.flac &>/dev/null; then
		INPUT_EXTENSION=flac
	fi
	if ls *.wv &>/dev/null; then
		INPUT_EXTENSION=wv
	fi
	if ls *.wav &>/dev/null; then
		INPUT_EXTENSION=wav
	fi
}

##############################################################################
# wrapper for unace binary, which is very limited
#
# First parameter should be the release to extract
# Second parameter should be the destination folder to extract the release
##############################################################################
_unace ()
{
	CURPATH=$(pwd)
	cd "$2"
	unace e "${CURPATH}/$1" &> /dev/null
	find ./ -type f -exec mv -f --backup=numbered "{}" . \;
	chmod -R 700 *
	rmdir */* &> /dev/null
	rmdir * &> /dev/null
	cd "$CURPATH"
}

##############################################################################
# extractRelease function
#
# It will extract a relase file to the specified folder
#
# First parameter should be the release to extract
# Second parameter should be the destination folder to extract the release
##############################################################################
extractRelease ()
{
	DECOMPRESS_OK="false"
	echo "Uncompressing Release..."
	unrar x -ep -or "$1" "$2" &> /dev/null #Try to uncompress as RAR file
	if [ "$?" -ne "0" ] ; then
		unzip -o -j "$1" -d "$2" &> /dev/null #If unrar fails, try to unzip the file
		if [ "$?" -ne "0" ] ; then
			_unace "$1" "$2" #If unzip also fails, try to unace the file
			if [ "$?" -ne "0" ] ; then
				error "Failed to uncompress the release. The file might be corrupted, or is not a valid RAR, ZIP or ACE file."
			else
				DECOMPRESS_OK="true"
			fi
		else
			DECOMPRESS_OK="true"
		fi
	else
		DECOMPRESS_OK="true"
	fi
}

##############################################################################
# createDefaultPreferences function
#
# It will create a preferences file based on a fixed template
##############################################################################
createDefaultPreferences ()
{
	cat > $SETTINGS_FILE << EOF
#User variables for audiommt

#Options used by lame to encode the MP3 files. Change to the one you like.
LAME_OPTIONS="--preset standard"

#Options used by mppenc to encode the MPC files. Change to the one you like.
MPPENC_OPTIONS="--extreme"

#Options used by oggenc to encode the OGG files. Change to the one you like.
OGGENC_OPTIONS="-q 5"

#Options used by mac to encode the APE files. Change to the one you like.
MAC_OPTIONS="-c3000"

#Options used by flac to encode the FLAC files. Change to the one you like.
FLAC_OPTIONS="-6"

#Options used by wavpack to encode the WV files. Change to the one you like.
WAVPACK_OPTIONS="-h"

# Folders used. It is highly recommended not to change them unless you have
# a strong reason to do so. If you change them, please notice:
# 1. DEST_DIR _must_ be different from TEMP_DIR
# 2. TEMP_DIR _must_ be used exclusively by audiommt.sh, since it is
#    deleted completely every time the program runs. Do not, under any 
#    circumstances, set TEMP_DIR to be $HOME, since all of its
#    contents will be deleted on exit without asking. You've been warned.
DEST_DIR="${HOME}/music"
TEMP_DIR="${HOME}/mmtemp"
EOF
	echo "Preferences file $SETTINGS_FILE created. Please check it and edit it to suit your taste."
}

##############################################################################
# lowercaseFiles function
#
# It will change all the uppercase letters found in the filenames extensions to 
# lowercase. This is done for every file in the folder.
##############################################################################
lowercaseFiles ()
{
	for FILE in *
	do
		OLDNAME=`echo "$FILE" | sed 's/\(.*\)\..*/\1/'`
		OLDEXT=`echo "$FILE" | sed 's/.*\.//'`
		NEWEXT=`echo "$OLDEXT" | tr "[A-Z]" "[a-z]"`
		NEWNAME=$OLDNAME.$NEWEXT
		mv "$FILE" "$NEWNAME" 2>/dev/null
	done
}

##############################################################################
# error function
#
# It is used to report errors that occur during the execution of the program
# both on screen and in the errors file
#
# First parameter should be the string to print as error
##############################################################################
error ()
{
	echo -en '\E[31m'
	echo -en '\E[5m'

	let "ERRORS += 1"

	echo ""
	echo "ERROR: $1"
	echo "ERROR: $1 -> $RELEASE" >> $LOG_FILE
	echo ""

	echo -en "\E[?25h"
	echo -en "\E[0m"
}

##############################################################################
# main function
#
# It will convert the given file into the selected mode
#
# First parameter should be the mode to process the file
# Second parameter is the file or folder to process
##############################################################################
main ()
{
	# Go to the home directory
	cd "${HOME}"

	#Check if the release file/directory can be read. If not, exit with error.
	if [ ! -r "$2" ]; then
		error "Error: File/directory $2 is not readable or does not exist."
	else

		#Automatically detect the release's year based on the filename, or set it to 0 if it can't be autodetected
		YEAR=`echo "$2" | sed 's/[0-9][0-9][0-9][0-9]/<<&>>/' | sed 's/^.*<<//' | sed 's/>>.*$//' | grep [0-9][0-9][0-9][0-9]`
		if [ "x$YEAR" = "x" ]; then
			ALBUM_PREFIX=""
		else
			ALBUM_PREFIX="($YEAR) "
		fi

		#Delete current temp folder (if exists) and create a new one
		rm -rf "${TEMP_DIR}" &>/dev/null
		mkdir -p "${TEMP_DIR}" &>/dev/null

		#Check if the release is a file or a folder. If folder, do not uncompress.
		DIRECTORY="false"
		if [ -d "$2" ]; then
			DIRECTORY="true"
		fi

		#Check if parameter is a file or a folder. If file, uncompress it.
		if [[ "$DIRECTORY" = "false" ]]; then # Not a folder 
			extractRelease "$2" "${TEMP_DIR}"
		else
			#It should be an already-uncompressed MMT release, since the parameter is a folder
			echo "Copying release source files to temp folder for processing..."
			cp -Rvf "$2"/* "${TEMP_DIR}" &>/dev/null
			if [ $? -eq 0 ]; then
				DECOMPRESS_OK="true"
			else
				DECOMPRESS_OK="false"
				error "Error copying release files from \"$2\" to \"${TEMP_DIR}\""
			fi
		fi

		#Only process files if the extraction was successful.
		if [[ "$DECOMPRESS_OK" = "true" ]]; then
			echo ""
			echo "Successfully extracted the release to folder ${TEMP_DIR}"
			echo ""

			#change to working dir
			cd "${TEMP_DIR}"
	
			# Lowercase the extensions of audio files. This avoids many issues with 
			# lower/uppercase, but it might create other issues. Let's see how it behaves...
			lowercaseFiles
	
			#Now check what kind of release this is.
			releasetype
			TYPE=$?

			#Check the format used to compress the release and set the INPUT_EXTENSION variable accordingly
			checkinputformat
		
			#Now comes the processing of the files. It will be done this way:
	
			#If the release is type 1 or 2, then process the audio IMAGE with the CUE
			#in the folder. In the case of releases type 2, any CUE will do, since they
			#should be the same, just different filename.
			if [[ $TYPE -eq 1 || $TYPE -eq 2 ]]; then
	
				#Get image filename
				IMAGE_FILE=`ls *.$INPUT_EXTENSION`

				#if there are several cue files, get just one, and ignore the rest
				i=0
				for C in *.cue
				do
					if [ $i -eq 0 ]; then
						CUE_FILE="$C"
						i=1
					else
						rm -f "$C"
					fi
				done
		
				#convert the image
				process "${CUE_FILE}" "${IMAGE_FILE}" "$OUTPUT_EXTENSION"
				moveExtraFiles
			fi
	
			#If the release is type 3 or 4, it should be asked for each IMAGE which one 
			#is its CUE file, then process them in pairs. It could be guessed from the 
			#filename...but I'd rather not :D
			if [[ $TYPE -eq 3 || $TYPE -eq 4 ]]; then
				for IMAGE_FILE in *.${INPUT_EXTENSION}
				do
					if [ -r "${IMAGE_FILE}.cue" ]; then
						CUEFILE="${IMAGE_FILE}.cue"
						echo "Processing image \"${IMAGE_FILE}\" with CUE file \"${CUEFILE}\""
						process "$CUEFILE" "${IMAGE_FILE}" $OUTPUT_EXTENSION
					elif [ -r "${IMAGE_FILE%.$INPUT_EXTENSION}.cue" ]; then
						CUEFILE="${IMAGE_FILE%.$INPUT_EXTENSION}.cue"
						echo "Processing image \"${IMAGE_FILE}\" with CUE file \"${CUEFILE}\""
						process "$CUEFILE" "${IMAGE_FILE}" $OUTPUT_EXTENSION
					else
						echo ""
						echo "***********************************************************************"
						echo "Could not detect the CUE file corresponding to file ${IMAGE_FILE}"
						echo "Please select the number of cue associated to the following image file:"
						echo " "
						echo "--> ${IMAGE_FILE}"
						echo " "
						echo "If there are several CUE files that fit, just choose any of them"
						echo ""
						select CUEFILE in *.cue "Ignore this file"
						do
							if [ "$CUEFILE" = "Ignore this file" ]; then
								echo "File $IMAGE_FILE ignored, it will not be processed"
							else
								process "$CUEFILE" "${IMAGE_FILE}" $OUTPUT_EXTENSION
							fi
							break
						done
					fi
				done
				moveExtraFiles
			fi
	
			#If the release is type 5, and it should be converted to tracks, then each audio 
			#file will be converted  with the same original name it had. No tagging
			#will be performed, since there is no way to know how the file is named, or if
			#the file name has any relation to the tags. Notice that at this moment tags from 
			#the original file are not imported, that could be done in the future.
			if [[ $TYPE -eq 5 ]]; then
				ALBUM_ARTIST="Untagged Albums"
				ALBUM_TITLE="$(basename "$RELEASE")"
				ALBUM_SUBDIR="$ALBUM_TITLE"
				mkdir -p "${DEST_DIR}/$ALBUM_ARTIST/$ALBUM_SUBDIR" &>/dev/null
				COMPRESS_ERRORS=0
				for FILE in *.$INPUT_EXTENSION
				do
					shnconv "$FILE" -o wav
					rm -f "$FILE"
					compress "${FILE%.$INPUT_EXTENSION}".wav "${DEST_DIR}/$ALBUM_ARTIST/$ALBUM_SUBDIR/${FILE%.$INPUT_EXTENSION}.${OUTPUT_EXTENSION}" ${OUTPUT_EXTENSION}
					if [ $? -ne 0 ]; then
        	                        	let COMPRESS_ERRORS+=1
					else
						rm -f "${FILE%.$INPUT_EXTENSION}".wav
	                	        fi
 				done
		                if [ $COMPRESS_ERRORS -gt 0 ]; then
		                        error "There were $COMPRESS_ERRORS tracks that had errors while compressing"
                		else
		                        let SUCCESS+=1
                		fi
				echo ""
				echo "Done. The resultant files are in ${DEST_DIR}/$ALBUM_ARTIST/$ALBUM_SUBDIR folder, but they have no tags."
				echo ""
				moveExtraFiles
			fi
	
			#If the release is type 6, it will be ignored, since we have no idea what 
			#it might be. Maybe it's some other RAR file or something like that.
			if [[ $TYPE -eq 6 ]]; then
				error "This file or folder does not seem to contain a valid MMT release. Ignoring it."
			fi
	
			#Finally, delete any files in the temp folder and the temp folder itself
			cleanup
		else
			error "Error extracting / copying the release"
		fi
	fi
}

##############################################################################
# Main program
##############################################################################

#File being worked on
RELEASE=""

#Set global variables
ERRORS=0
SUCCESS=0
LOG_FILE=${HOME}/audiommt-log.txt

#Try to detect the required tools, and issue an error if a required one is missing
#or a warning if an optional one is missing
REQTOOLS="sed seq bash flac unrar wvunpack mac cuebreakpoints shnsplit lame id3 vorbiscomment metaflac"
REQCOMPLETE=1
OPTTOOLS="unzip unace"
for COMMAND in $REQTOOLS; do
	which $COMMAND &> /dev/null
	if [ ! $? = 0 ]; then
		error "Command "$COMMAND" not found. Install it to continue."
		REQCOMPLETE=0
	fi
done
if [ $REQCOMPLETE = 0 ]; then
	exit 1
fi

#Start the program by setting the options file
SETTINGS_FILE="${HOME}/.audiommtrc"

#By default, no extension in the input audio files is assumed
INPUT_EXTENSION=""

#Set the conversion mode to none
OUTPUT_EXTENSION=""

#Assume the release is compressed
DIRECTORY="false"

#Prefix of the destination subdirectory
ALBUM_PREFIX=""
DECOMPRESS_OK="false"

#Check for the preferences file, and create a default if there is none.
if [ -r $SETTINGS_FILE ]; then
	source $SETTINGS_FILE
else
	createDefaultPreferences
	exit 2
fi

#Check the mode (output extension) and set it as global variable
if [[ "$1" = "-mp3" || "$1" = "-mpc" || "$1" = "-ogg" || "$1" = "-ape" || "$1" = "-flac" || "$1" = "-wv" ]]; then
	OUTPUT_EXTENSION=$(echo $1 | sed 's/-//g')
else
	usage
	exit 3
fi     

#Set the output folder according to the output file type
DEST_DIR="${DEST_DIR}/$OUTPUT_EXTENSION"

# Set the current folder as initial folder
INIT_DIR=$(pwd)

# Drop the mode from the parameters list
shift

# Set the start date on the error file
echo "Start: $(date +%Y-%m-%d\ %H:%M.%S)" >> $LOG_FILE

# Loop over the rest of parameters until there are no more.
# All the remaining parameters should be files to convert.
while [[ ! x"$1" = x"" ]]; do
	RELEASE="$1"
	echo "*****************************************************************************"
	echo "*** Starting processing of file "$RELEASE" ***"
	echo ""
	main $OUTPUT_EXTENSION "$RELEASE"
	echo ""
	echo "*** Finished processing of file "$RELEASE" ***"
	echo "*****************************************************************************"
	shift
done

#Report total number of errors
echo ""
echo "There were $SUCCESS releases processed successfully and $ERRORS releases with problems during processing"
echo "There were $SUCCESS releases processed successfully and $ERRORS releases with problems during processing" >> $LOG_FILE
if [[ ! x"$ERRORS" = x"0" ]]; then
	echo "Check the file $LOG_FILE to see the errors found"
fi
echo ""

# Set the end date on the error file
echo "End: $(date +%Y-%m-%d\ %H:%M.%S)" >> $LOG_FILE
echo "------------------------------------------------------------------------------------------" >> $LOG_FILE

# Return to the directory where executed initially
cd ${INIT_DIR}

#Exit normally.
exit 0

